#include "Entity.h"
#include "LD37.h"

extern LD37* game;
extern int ENTCOUNT;

Entity::Entity(ALLEGRO_BITMAP* g)
{
	graphic = g;
	killMe = false;
	solid = true;
	liftable = false;
	thrown = false;
	thx = 0;
	thy = 0;
	thdmg = 2;
	killMe = false;
}

void Entity::Update()
{
	px = x;
	py = y;
	tx = px * 16;
	ty = py * 16;
}

void Entity::HandleTransition()
{
	if (px != x || py != y)
	{
		tx += ((x * 16) - tx) / 2.0f;
		ty += ((y * 16) - ty) / 2.0f;
	}
	else
	{
		tx = x * 16;
		ty = y * 16;
	}
}

void Entity::Render()
{
	al_draw_bitmap_region(graphic, srcx, srcy, 16, 16, x*16, y*16, 0);
}

Entity::~Entity()
{
	graphic = nullptr;
}

bool Entity::CanWalk(int xx, int yy, bool dodgeBarrel, bool includePlayer)
{
	if (game->board[xx][yy] != 1) return false;
	for (int i = 0; i < ENTCOUNT; ++i)
	{
		if (game->ents[i] != nullptr && game->ents[i] != this)
		{
			if (game->ents[i]->solid && game->ents[i]->x == xx && game->ents[i]->y == yy && (dodgeBarrel || game->ents[i]->thrown == false))
			{
				return false;
			}
		}
	}
	if (includePlayer && xx == game->ply->x && yy == game->ply->y)
	{
		return false;
	}
	return true;
}

bool Entity::TryMove(int dx, int dy) //Returns true if hit something
{
	if (game->board[x + dx][y + dy] == 1)
	{
		for (int i = 0; i < ENTCOUNT; ++i)
		{
			if (game->ents[i] != nullptr) 
			{
				if (game->ents[i]->solid && game->ents[i] != this) 
				{
					if ((game->ents[i]->x == x + dx && game->ents[i]->y == y + dy)
						|| (game->ents[i]->x == x && game->ents[i]->y == y))
					{
						if (game->ents[i]->thrown == false || (game->ents[i]->thx != thx && game->ents[i]->thy != thy)) {
							game->ents[i]->Bump(this);
							Bump(game->ents[i]);
							return true;
						}
					}
				}
			}
		}
		if (x + dx == game->ply->x && y + dy == game->ply->y)
		{
			game->ply->Bump(this);
			//Bump(game->ply);
			return true;
		}
		x += dx;
		y += dy;
		return false;
	}
	return true;
}

void Entity::Bump(Entity* perp)
{
}