#include <iostream>
#include <allegro5\allegro.h>
#include <allegro5\allegro_acodec.h>
#include <allegro5\allegro_audio.h>
#include <allegro5\allegro_font.h>
#include <allegro5\allegro_image.h>
#include <allegro5\allegro_primitives.h>
#include "LD37.h"

using namespace std;

//Music

LD37* game;
int state = 0;
ALLEGRO_FONT* font;
ALLEGRO_SAMPLE_ID* music;
ALLEGRO_SAMPLE* musicSample;

int main(int argc, char **argv)
{
	game = new LD37();

	al_init();
	al_install_audio();
	al_install_keyboard();
	al_init_image_addon();
	al_init_primitives_addon();
	al_init_font_addon();
	al_init_acodec_addon();

	int got = 0; //Game Over Timer
	
	ALLEGRO_DISPLAY* disp = al_create_display(960, 720);
	if (disp == NULL) cout << "DISPLAY DUN GOOFED" << endl;
	al_set_window_title(disp, "LD37");

	ALLEGRO_TIMER* time = al_create_timer(1.0/60.0);
	if (time == NULL) cout << "TIMER DUN GOOFED" << endl;

	ALLEGRO_EVENT_QUEUE* eq = al_create_event_queue();
	if (eq == NULL) cout << "EVENT QUEUE DUN GOOFED" << endl;
	
	al_register_event_source(eq, al_get_timer_event_source(time));
	al_register_event_source(eq, al_get_display_event_source(disp));
	al_register_event_source(eq, al_get_keyboard_event_source());

	al_start_timer(time);

	font = al_create_builtin_font();
	al_reserve_samples(16);
	musicSample = al_load_sample("assets/music.wav");
	if (musicSample == NULL)
	{
		cout << "The music refused to load. Please contact the developer." << endl;
	}
	game->Init();

	ALLEGRO_BITMAP* rt = al_create_bitmap(320, 240); //This is for the non-game states to use
	ALLEGRO_BITMAP* title = al_load_bitmap("assets/title.tga");
	ALLEGRO_BITMAP* tscreen = al_load_bitmap("assets/titlescreen.bmp");
	ALLEGRO_BITMAP* inscreen = al_load_bitmap("assets/instructions.bmp");
	ALLEGRO_BITMAP* story = al_load_bitmap("assets/story.bmp");
	int tt = 0;
	int et = 0;

	

	
	
	ALLEGRO_SAMPLE* startup = al_load_sample("assets/start.wav");
	ALLEGRO_SAMPLE* gameover[4];
	gameover[0] = al_load_sample("assets/gameover.wav");
	gameover[1] = al_load_sample("assets/gameover1.wav");
	gameover[2] = al_load_sample("assets/gameover2.wav");
	gameover[3] = al_load_sample("assets/gameover3.wav");

	bool quit = false;
	while (!quit)
	{
		ALLEGRO_EVENT evnt;
		while (al_get_next_event(eq, &evnt))
		{
			if (evnt.type == ALLEGRO_EVENT_TIMER)
			{
				switch (state) 
				{
				case 0: //Title Screen
					al_set_target_bitmap(rt);
					al_clear_to_color(al_map_rgb(0,0,0));
					tt += 1;
					al_draw_bitmap(tscreen, 0, 0, 0);
					al_draw_bitmap(title, 0, sin(tt*0.1f)*4.0f+4, 0);

					al_set_target_bitmap(al_get_backbuffer(disp));
					al_draw_scaled_bitmap(rt, 0, 0, 320, 240, 0, 0, 960, 720, 0);
					break;
				case 1:
					game->Update();
					al_set_target_bitmap(al_get_backbuffer(disp));
					al_draw_scaled_bitmap(game->renderTarget, 0, 0, 320, 240, 0, 0, 960, 720, 0);
					break;
				case 2: //Game Over Screen
					if (got == 0)
					{
						al_stop_samples();
						int r = rand() % 4;
						al_play_sample(gameover[r], 1.0f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0);
					}
					got += 1;
					al_set_target_bitmap(rt);
					al_clear_to_color(al_map_rgb(0, 0, 0));

					al_draw_text(font, al_map_rgb(255, 0, 0), 160, 120, ALLEGRO_ALIGN_CENTER, "GAME OVER");

					al_set_target_bitmap(al_get_backbuffer(disp));
					al_draw_scaled_bitmap(rt, 0, 0, 320, 240, 0, 0, 960, 720, 0);
					if (got > 200)
					{
						got = 0;
						game->DeInit();
						delete game;
						game = new LD37();
						game->Init();
						state = 1;
						al_stop_samples();
						al_play_sample(musicSample, 1.0f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_LOOP, music);
						al_play_sample(startup, 1.0f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0);
					}
					
					break;
				case 3: //Instructions
					al_set_target_bitmap(rt);
					al_clear_to_color(al_map_rgb(0, 0, 0));
					al_draw_bitmap(inscreen, 0, 0, 0);
					al_set_target_bitmap(al_get_backbuffer(disp));
					al_draw_scaled_bitmap(rt, 0, 0, 320, 240, 0, 0, 960, 720, 0);
					break;
				case 4: //Story
					al_set_target_bitmap(rt);
					al_clear_to_color(al_map_rgb(0, 0, 0));
					al_draw_bitmap(story, 0, 0, 0);
					al_set_target_bitmap(al_get_backbuffer(disp));
					al_draw_scaled_bitmap(rt, 0, 0, 320, 240, 0, 0, 960, 720, 0);
					break;
				case 5: //Ending
					et += 1;
					if (et > 750)
					{
						et = 0;
						game->DeInit();
						delete game;
						game = new LD37();
						game->Init();
						state = 0;
					}
					al_set_target_bitmap(rt);
					al_clear_to_color(al_map_rgb(0, 0, 0));
					
					int ayy = floor(sin(et * 0.05f) * 128.0f) + 128;

					al_draw_text(font, al_map_rgb(255, ayy, ayy), 8, 16, 0, "THE CRYSTALS OF GINGLEDOOF ARE YOURS!");
					al_draw_text(font, al_map_rgb(255, ayy, ayy), 8, 32, 0, "NOW THAT YOU CAN FINANCE YOUR PORTAL");
					al_draw_text(font, al_map_rgb(255, ayy, ayy), 8, 48, 0, "TO HELL, YOU WILL UNLEASH DEMONIC");
					al_draw_text(font, al_map_rgb(255, ayy, ayy), 8, 64, 0, "FORCES UPON THIS WRETCHED PLANET AND");
					al_draw_text(font, al_map_rgb(255, ayy, ayy), 8, 80, 0, "RULE AS A GOD FOR ETERNITY!");

					al_draw_text(font, al_map_rgb(000, 000, 255), 8, 192, 0, "THANKS FOR PLAYING.");

					al_set_target_bitmap(al_get_backbuffer(disp));
					al_draw_scaled_bitmap(rt, 0, 0, 320, 240, 0, 0, 960, 720, 0);
					break;
				}
				al_flip_display();
			}
			if (evnt.type == ALLEGRO_EVENT_KEY_DOWN)
			{
				game->keys[evnt.keyboard.keycode] = true;
				if (state == 0) 
				{
					if (evnt.keyboard.keycode == ALLEGRO_KEY_ENTER)
					{
						state = 1;
						al_stop_samples();
						al_play_sample(startup, 1.0f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0);
						al_play_sample(musicSample, 1.0f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_LOOP, music);
					}
					if (evnt.keyboard.keycode == ALLEGRO_KEY_H)
					{
						state = 3;
					}
					if (evnt.keyboard.keycode == ALLEGRO_KEY_S)
					{
						state = 4;
					}
				}
				else if (state == 3)
				{
					if (evnt.keyboard.keycode == ALLEGRO_KEY_H)
					{
						state = 0;
					}
				}
				
			}
			if (evnt.type == ALLEGRO_EVENT_KEY_CHAR)
			{
				game->key = evnt.keyboard.keycode;
			}
			if (evnt.type == ALLEGRO_EVENT_KEY_UP)
			{
				game->keys[evnt.keyboard.keycode] = false;
				if (state == 3 || state == 4) state = 0;
				if (evnt.keyboard.keycode == ALLEGRO_KEY_ESCAPE)
				{
					if (state == 1)
					{
						state = 0;
					}
					else if (state == 0)
					{
						quit = true;
					}
					else if (state == 3 || state == 4)
					{
						state = 0;
					}
				}
			}
			if (evnt.type == ALLEGRO_EVENT_DISPLAY_CLOSE)
			{
				quit = true;
				break;
			}
		}
	}
	
	game->DeInit();

	delete game;
	al_stop_samples();
	//free(game);
	//delete evnt;
	al_destroy_sample(startup);
	al_destroy_sample(musicSample);
	for (int i = 0; i < 4; ++i)
	{
		al_destroy_sample(gameover[i]);
	}
	//al_destroy_font(font);
	al_destroy_bitmap(story);
	al_destroy_bitmap(title);
	al_destroy_bitmap(inscreen);
	al_destroy_bitmap(tscreen);
	al_destroy_font(font);
	al_destroy_bitmap(rt);
	al_destroy_timer(time);
	al_destroy_event_queue(eq);
	al_destroy_display(disp);
	al_uninstall_audio();
	al_uninstall_keyboard();

	return 0;
}